"""
=================================
act.corrections (act.corrections)
=================================

.. currentmodule:: act.corrections

The procedures in this module contain corrections for various ARM datasets.

.. autosummary::
    :toctree: generated/

    ceil.correct_ceil
"""

from . import common
from . import ceil
from . import mpl
